/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file videoPlayer.cpp
@ingroup ExamplePrograms
@brief Simple example of image input/output from a video file, to a canvas window

This program will display in an image canvas the contents of a video file or the input from a video resource.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./video-player --URL=<path or url to video>
\endcode

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@author PARP Research Group. University of Murcia, Spain.
*/

//#include <stdio.h>
//#include <stdlib.h>
//#include <iostream>

#include <QVApplication>
#include <QVMPlayerCamera>
#include <QVGUI>
#include <QVImageCanvas>

#ifndef DOXYGEN_IGNORE_THIS
class PlayerWorker: public QVWorker
	{
	public:
		PlayerWorker(QString name): QVWorker(name)
			{
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			}

		void iterate() { }
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		"Example program for QVision library. Displays the contents of a video source."
		);

	QVMPlayerCamera camera("Video");
	PlayerWorker worker("Video player worker");
	camera.link(&worker,"Input image");

	QVGUI interface;

	QVImageCanvas imageCanvas("Output image");
	imageCanvas.linkProperty(worker, "Input image");

	return app.exec();
	}

#endif
