/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef QV3DMODEL_H
#define QV3DMODEL_H

#ifndef DOXYGEN_IGNORE_THIS
#include <QGLWidget>

#include <qvcore/qvimage.h>

//////////////////////////////////////////////////////////////
class QV3DModelSegment
	{
	friend class QV3DModel;

	public:
		QV3DModelSegment(	const double x0, const double y0, const double z0,
					const double x1, const double y1, const double z1,
					const double r, const double g, const double b)
				{
				this->x0 = x0;	this->y0 = y0;	this->z0 = z0;
				this->x1 = x1;	this->y1 = y1;	this->z1 = z1;
				this->r = r;	this->g = g;	this->b = b;
				}
	private:
		double x0, y0, z0, x1, y1, z1, r, g, b;	
	};

class QV3DModelText
	{
	friend class QV3DModel;

	public:
		QV3DModelText(const double x, const double y, const double z, const QString &text, const double r, const double g, const double b)
				{
				this->x = x;	this->y = y;	this->z = z;
				this->r = r;	this->g = g;	this->b = b;
				this->text = text;
				}
	private:
		double x, y, z;
		double r, g, b;
		QString text;
	};

class QVGLCanvas;
class QV3DModel
	{
	private:
		QList<QV3DModelSegment> segmentList;
		QList<QV3DModelText> textList;
		QList< QVImage<uChar> > imgList;

	public:
		QV3DModel()	{}

		void addSegment(	const double x0, const double y0, const double z0,
					const double x1, const double y1, const double z1,
					const double r, const double g, const double b)
					{ segmentList.append(QV3DModelSegment(x0, y0, z0, x1, y1, z1, r, g, b)); }

		void addText(const int x, const int y, const int z, const QString &text, const double r, const double g, const double b)
			{ textList.append(QV3DModelText(x,y,z,text, r,g,b)); }

		void addImage(QVImage<uChar,1> image)	{ imgList.append(image); }

		virtual void paint(QVGLCanvas &glWidget) const;

		static QV3DModel image(QVImage<uChar> &);
		static QV3DModel crossHair(const double size);
		static QV3DModel cube(const double size);
		static QV3DModel cuboid(const double sizex, const double sizey, const double sizez);
		static QV3DModel referenceCoordinates(const double size, const bool labelAxis = false);
		static QV3DModel grid(const double stepX, const double stepY, const int cellsX, const int cellsY);
	};

Q_DECLARE_METATYPE(QV3DModel);

#endif

#endif


