/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvparamsinspectorwidget.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvparamsinspectorwidget.h>

#include <QtGui/QWidget>

#include <qvcore/qvpropertycontainer.h>
#include <qvcore/qvworker.h>
#include <qvgui/qvparamwidget.h>
#include <qvdta/qvindexedstringlist.h>

QVParamsInspectorWidget::QVParamsInspectorWidget(QVPropertyContainer *holder, QWidget *parent):QWidget(parent)
	{
	//setGeometry(QRect(10, 20, 200, 0));
	vboxLayout = new QVBoxLayout(this);
	vboxLayout->setSpacing(0);

	QVWorker *worker;
	if((worker = dynamic_cast<QVWorker*>(holder)) != NULL)
		foreach(QString property, worker->getTriggerList())
			vboxLayout->addWidget(new QVWorkerTriggerWidget(worker, property, this));

	foreach(QString property, holder->getPropertyList())
		if(holder->isLinkedInput(property) or not holder->isInput(property) or holder->isGUIInvisible(property))
			continue;
		else if(holder->isType<int>(property)) // if the property is int.
			{
			QVIntParamWidget *int_widget = new QVIntParamWidget(holder,this,property);
			vboxLayout->addWidget(int_widget);
			writeOutputProperties();
			connect(int_widget,SIGNAL(valueChanged(int)),this,SLOT(somePropertyChanged()));
			}
		else if(holder->isType<double>(property)) // if the property is double.
			{
			QVDoubleParamWidget *double_widget = new QVDoubleParamWidget(holder,this,property);
			vboxLayout->addWidget(double_widget);
			writeOutputProperties();
			connect(double_widget,SIGNAL(valueChanged(double)),this,SLOT(somePropertyChanged()));
			}
		else if(holder->isType<bool>(property)) // if the property is bool.
			{
			QVBoolParamWidget *bool_widget = new QVBoolParamWidget(holder,this,property);
			vboxLayout->addWidget(bool_widget);
			writeOutputProperties();
			connect(bool_widget,SIGNAL(valueChanged(bool)),this,SLOT(somePropertyChanged()));
			}
		else if(holder->isType<QString>(property)) // if the property is QString.
			{
			QVStringParamWidget *st_widget = new QVStringParamWidget(holder,this,property);
			vboxLayout->addWidget(st_widget);
			writeOutputProperties();
			connect(st_widget,SIGNAL(valueChanged(QString)),this,SLOT(somePropertyChanged()));
			}
		else if(holder->isType<QVIndexedStringList>(property)) // if the property is QVIndexedStringList.
			{
			QVStringListParamWidget *sl_widget = new QVStringListParamWidget(holder,this,property);
			vboxLayout->addWidget(sl_widget);
			writeOutputProperties();
			connect(sl_widget,SIGNAL(valueChanged(QVIndexedStringList)),this,SLOT(somePropertyChanged()));
			}

	vboxLayout->addStretch();
	}

void QVParamsInspectorWidget::somePropertyChanged()
	{
	writeOutputProperties();
	}
