/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvparamwidget.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvparamwidget.h>

#include <QLineEdit>
#include <QSlider>
#include <QLabel>
#include <QCheckBox>

#include <QVBoxLayout>
#include <QHBoxLayout>

#include <qwt_slider.h>




QVIntParamWidget::QVIntParamWidget(QVPropertyContainer *_orig_holder, QVPropertyContainer *_gui_holder, const QString _property, QWidget *parent): QWidget(parent), orig_holder(_orig_holder), gui_holder(_gui_holder), property(_property)
	{
	value = orig_holder->getPropertyValue<int>(property);
	if(orig_holder->hasRange(property))
		{
		max = orig_holder->getPropertyMaximum<int>(property);
		min = orig_holder->getPropertyMinimum<int>(property);
		gui_holder->addProperty<int>(property+"_internal_gui",QVPropertyContainer::outputFlag,value,"",min,max);
		gui_holder->linkProperty(property+"_internal_gui",orig_holder,property,QVPropertyContainer::AsynchronousLink);

		lineedit = new QLineEdit(this);
		lineedit->setFixedWidth(80);
		slider = new QSlider(Qt::Horizontal,this);
		slider->setMinimum(min);
		slider->setMaximum(max);
		slider->setFixedWidth(150);

		QVBoxLayout *vboxlayout = new QVBoxLayout(this);
		vboxlayout->setSpacing(0);
		QHBoxLayout *hboxlayout1 = new QHBoxLayout();
		hboxlayout1->addWidget(new QLabel(QString("<i>int</i> <b>%1</b>").arg(property)));
		hboxlayout1->addStretch();
		hboxlayout1->addWidget(new QLabel(QString("(%1,%2)").arg(min).arg(max)));
		QHBoxLayout *hboxlayout2 = new QHBoxLayout();
		hboxlayout2->addWidget(lineedit);
		hboxlayout2->addStretch();
		hboxlayout2->addWidget(slider);
		vboxlayout->addLayout(hboxlayout1);
		vboxlayout->addLayout(hboxlayout2);

		slider->setValue(value);
		connect(slider,SIGNAL(valueChanged(int)),this,SLOT(setValue()));
		}
	else
		{
		gui_holder->addProperty<int>(property+"_internal_gui",QVPropertyContainer::outputFlag,value,"");
		gui_holder->linkProperty(property+"_internal_gui",orig_holder,property,QVPropertyContainer::AsynchronousLink);

		lineedit = new QLineEdit(this);
		lineedit->setFixedWidth(80);

		QHBoxLayout *hboxlayout = new QHBoxLayout(this);
		hboxlayout->addWidget(new QLabel(QString("<i>int</i> <b>%1</b>").arg(property)));
		hboxlayout->addWidget(lineedit);
		hboxlayout->addStretch();
		}

	connect(lineedit,SIGNAL(editingFinished()),this,SLOT(setValue()));
	gui_holder->setPropertyValue<int>(property+"_internal_gui",value);
	lineedit->setText(QString("%1").arg(value));
	emit valueChanged(value);
	}

void QVIntParamWidget::setValue()
	{
	if(sender() == lineedit)
		{
		bool ok;
		value = lineedit->text().toInt(&ok);
		if( (not ok) or
			(orig_holder->hasRange(property) and (value<min or value>max) ) )
			value = gui_holder->getPropertyValue<int>(property+"_internal_gui");
		}
	else if(sender() == slider) {
		value = slider->value();
		}
	else
		value = gui_holder->getPropertyValue<int>(property+"_internal_gui");

	lineedit->setText(QString("%1").arg(value));
	if(orig_holder->hasRange(property))
		slider->setValue(value);
	gui_holder->setPropertyValue<int>(property+"_internal_gui",value);
	emit valueChanged(value);
	}


QVDoubleParamWidget::QVDoubleParamWidget(QVPropertyContainer *_orig_holder, QVPropertyContainer *_gui_holder, const QString _property, QWidget *parent): QWidget(parent), orig_holder(_orig_holder), gui_holder(_gui_holder), property(_property)
	{

	value = orig_holder->getPropertyValue<double>(property);
	if(orig_holder->hasRange(property))
		{
		max = orig_holder->getPropertyMaximum<double>(property);
		min = orig_holder->getPropertyMinimum<double>(property);
		gui_holder->addProperty<double>(property+"_internal_gui",QVPropertyContainer::outputFlag,value,"",min,max);
		gui_holder->linkProperty(property+"_internal_gui",orig_holder,property,QVPropertyContainer::AsynchronousLink);

		lineedit = new QLineEdit(this);
		lineedit->setFixedWidth(80);
		qwtslider = new QwtSlider(this,Qt::Horizontal,QwtSlider::NoScale,QwtSlider::BgSlot);
		qwtslider->setThumbLength(20);
		qwtslider->setThumbWidth(10);
		qwtslider->setRange(min,max);
		qwtslider->setFixedWidth(150);

		QVBoxLayout *vboxlayout = new QVBoxLayout(this);
		vboxlayout->setSpacing(0);
		QHBoxLayout *hboxlayout1 = new QHBoxLayout();
		hboxlayout1->addWidget(new QLabel(QString("<i>double</i> <b>%1</b>").arg(property)));
		hboxlayout1->addStretch();
		hboxlayout1->addWidget(new QLabel(QString("(%1,%2)").arg(min).arg(max)));
		QHBoxLayout *hboxlayout2 = new QHBoxLayout();
		hboxlayout2->addWidget(lineedit);
		hboxlayout2->addStretch();
		hboxlayout2->addWidget(qwtslider);
		vboxlayout->addLayout(hboxlayout1);
		vboxlayout->addLayout(hboxlayout2);

		qwtslider->setValue(value);
		connect(qwtslider,SIGNAL(valueChanged(double)),this,SLOT(setValue()));
		}
	else
		{
		gui_holder->addProperty<double>(property+"_internal_gui",QVPropertyContainer::outputFlag,value,"");
		gui_holder->linkProperty(property+"_internal_gui",orig_holder,property,QVPropertyContainer::AsynchronousLink);

		lineedit = new QLineEdit(this);
		lineedit->setFixedWidth(80);

		QHBoxLayout *hboxlayout = new QHBoxLayout(this);
		hboxlayout->addWidget(new QLabel(QString("<i>double</i> <b>%1</b>").arg(property)));
		hboxlayout->addWidget(lineedit);
		hboxlayout->addStretch();
		}

	connect(lineedit,SIGNAL(editingFinished()),this,SLOT(setValue()));
	gui_holder->setPropertyValue<double>(property+"_internal_gui",value);
	lineedit->setText(QString("%1").arg(value));
	emit valueChanged(value);

	}

void QVDoubleParamWidget::setValue()
	{
	if(sender() == lineedit)
		{
		bool ok;
		value = lineedit->text().toDouble(&ok);
		if( (not ok) or
			(orig_holder->hasRange(property) and (value<min or value>max) ) )
			value = gui_holder->getPropertyValue<double>(property+"_internal_gui");
		}
	else if(sender() == qwtslider) {
		value = qwtslider->value();
		}
	else
		value = gui_holder->getPropertyValue<double>(property+"_internal_gui");

	lineedit->setText(QString("%1").arg(value));
	if(orig_holder->hasRange(property))
		qwtslider->setValue(value);
	gui_holder->setPropertyValue<double>(property+"_internal_gui",value);
	emit valueChanged(value);
	}

QVBoolParamWidget::QVBoolParamWidget(QVPropertyContainer *_orig_holder, QVPropertyContainer *_gui_holder, const QString _property, QWidget *parent): QWidget(parent), orig_holder(_orig_holder), gui_holder(_gui_holder), property(_property)
	{
	QHBoxLayout *hboxlayout = new QHBoxLayout(this);
	//hboxlayout->setSpacing(0);
	QLabel *label = new QLabel(QString("<i>bool</i> <b>%1</b>").arg(property),this);
	hboxlayout->addWidget(label);
	checkbox = new QCheckBox(this);
	hboxlayout->addWidget(checkbox);

	value = orig_holder->getPropertyValue<bool>(property);

	gui_holder->addProperty<bool>(property+"_internal_gui",QVPropertyContainer::outputFlag,value,"");

	gui_holder->linkProperty(property+"_internal_gui",orig_holder,property,QVPropertyContainer::AsynchronousLink);

	connect(checkbox,SIGNAL(stateChanged(int)),this,SLOT(setValue()));
	gui_holder->setPropertyValue<bool>(property+"_internal_gui",value);
	if(value)
		checkbox->setCheckState(Qt::Checked);
	else
		checkbox->setCheckState(Qt::Unchecked);
	emit valueChanged(value);
	}

void QVBoolParamWidget::setValue()
	{
	if (checkbox->checkState() == Qt::Unchecked)
		value = false;
	else if (checkbox->checkState() == Qt::Checked)
		value = true;

	gui_holder->setPropertyValue<bool>(property+"_internal_gui",value);
	emit valueChanged(value);
	}

QVStringParamWidget::QVStringParamWidget(QVPropertyContainer *_orig_holder, QVPropertyContainer *_gui_holder, const QString _property, QWidget *parent): QWidget(parent), orig_holder(_orig_holder), gui_holder(_gui_holder), property(_property)
	{
	QHBoxLayout *hboxlayout = new QHBoxLayout(this);
	hboxlayout->setSpacing(0);
	QLabel *label = new QLabel(QString("<i>string</i> <b>%1</b>").arg(property),this);
	hboxlayout->addWidget(label);
	lineedit = new QLineEdit(this);
	hboxlayout->addWidget(lineedit);

	value = orig_holder->getPropertyValue<QString>(property);

	gui_holder->addProperty<QString>(property+"_internal_gui",QVPropertyContainer::outputFlag,value,"");
	gui_holder->linkProperty(property+"_internal_gui",orig_holder,property,QVPropertyContainer::AsynchronousLink);

	connect(lineedit,SIGNAL(textChanged(QString)),this,SLOT(setValue()));

	gui_holder->setPropertyValue<QString>(property+"_internal_gui",value);

	emit valueChanged(value);
	}

void QVStringParamWidget::setValue()
	{
	value = lineedit->text();
	gui_holder->setPropertyValue<QString>(property+"_internal_gui",value);
	emit valueChanged(value);
	}


QVStringListParamWidget::QVStringListParamWidget(QVPropertyContainer *_orig_holder, QVPropertyContainer *_gui_holder, const QString _property, QWidget *parent): QWidget(parent), orig_holder(_orig_holder), gui_holder(_gui_holder), property(_property)
	{
	QHBoxLayout *hboxlayout = new QHBoxLayout(this);
	hboxlayout->setSpacing(0);
	QLabel *label = new QLabel(QString("<i>stringlist</i> <b>%1</b>").arg(property),this);
	hboxlayout->addWidget(label);
	combobox = new QComboBox(this);
	hboxlayout->addWidget(combobox);

	value = orig_holder->getPropertyValue<QVIndexedStringList>(property);

	gui_holder->addProperty<QVIndexedStringList>(property+"_internal_gui",QVPropertyContainer::outputFlag,value,"");
	gui_holder->linkProperty(property+"_internal_gui",orig_holder,property,QVPropertyContainer::AsynchronousLink);

	connect(combobox,SIGNAL(activated(int)),this,SLOT(setValue()));

	gui_holder->setPropertyValue<QVIndexedStringList>(property+"_internal_gui",value);

	combobox->addItems(value);
	combobox->setCurrentIndex(value.getIndex());
	emit valueChanged(value);
	}

void QVStringListParamWidget::setValue()
	{
	value.setIndex(combobox->currentIndex());
	gui_holder->setPropertyValue<QVIndexedStringList>(property+"_internal_gui",value);
	emit valueChanged(value);
	}

QVWorkerTriggerWidget::QVWorkerTriggerWidget(QVWorker *worker, const QString triggername, QWidget *parent): QWidget(parent), worker(worker), triggername(triggername)
	{
	QHBoxLayout *hboxlayout = new QHBoxLayout(this);
	toolbutton = new QToolButton(this);
	toolbutton->setText(triggername);
	hboxlayout->addWidget(new QLabel(QString("<i>trigger</i> <b>%1</b>").arg(triggername)));
hboxlayout->addStretch();
	hboxlayout->addWidget(toolbutton);

	connect(toolbutton,SIGNAL(pressed()),this,SLOT(setValue()));
	connect(this,SIGNAL(valueChanged(QString)),worker,SLOT(processTrigger(QString)));
	}

void QVWorkerTriggerWidget::setValue()
	{
	emit valueChanged(triggername);
	}

/*		doublespinbox->setMinimum(min);
		doublespinbox->setMaximum(max);
		int signif=static_cast<int>(round(log(max-min)/log(10)));
		double val=pow(10,signif);
		std::cout << val << " " << signif << std::endl;
		doublespinbox->setSingleStep(val/100);
		if(signif >= 5)
			doublespinbox->setDecimals(0);
		else
			doublespinbox->setDecimals(5-signif);
*/
